@@ -11,11 +11,15 @@ else:
     conditional_fuse = fuse
 
 @conditional_fuse
-def fused_rmsprop_step(param, grad, sq_avg, lr, beta, eps):
-    # Update running average of squared gradients and parameter in one kernel
-    sq_avg[:] = beta * sq_avg + (1 - beta) * grad * grad
-    param[:] -= lr * grad / (xp.sqrt(sq_avg) + eps)
-    return param
+def fused_update_momentum(momentum, grad, beta):
+    # momentum = beta * momentum + (1 - beta) * grad^2
+    return beta * momentum + (1 - beta) * grad * grad
+
+@conditional_fuse
+def fused_param_update(param, grad, lr, momentum, eps):
+    # param -= lr * grad / (sqrt(momentum) + eps)
+    denom = xp.sqrt(momentum) + eps
+    return param - lr * grad / denom
 
 
 class RMSprop(Optimizer):
@@ -46,15 +50,18 @@ class RMSprop(Optimizer):
         """
         for i, (name, param) in enumerate(self.params):
             if param.requires_grad and param.grad is not None:
-                
+                grad = param.grad
                 momentum_value = self.momentum[i][1]
                 
                 # Weight decay fused with grad (in-place)
                 if self.weight_decay > 0:
-                    xp.add(param.grad, self.weight_decay * param.data, out=param.grad)
+                    xp.add(grad, self.weight_decay * param.data, out=grad)
                 
-                # Fused squared-average + parameter update
-                param.data = fused_rmsprop_step(param.data, param.grad, momentum_value, self.lr, self.beta, self.eps)
+                # Fused momentum update
+                momentum_value[:] = fused_update_momentum(momentum_value, grad, self.beta)
+                
+                # Fused parameter update
+                param.data[:] = fused_param_update(param.data, grad, self.lr, momentum_value, self.eps)
 
     
     def state_dict(self) -> dict:
@@ -62,7 +69,6 @@ class RMSprop(Optimizer):
             "lr": self.lr,
             "beta": self.beta,
             "eps": self.eps,
-            "weight_decay": self.weight_decay,
             "params": self.params,
             "momentum": self.momentum,
         }
@@ -71,7 +77,6 @@ class RMSprop(Optimizer):
         self.lr = state_dict["lr"]
         self.beta = state_dict["beta"]
         self.eps = state_dict["eps"]
-        self.weight_decay = state_dict["weight_decay"]
         self.params = state_dict["params"]
         self.momentum = state_dict["momentum"]
     
@@ -79,4 +84,4 @@ class RMSprop(Optimizer):
         return f"RMSprop(lr={self.lr}, beta={self.beta}, eps={self.eps}, weight_decay={self.weight_decay})."
     
     def __str__(self) -> str:
-        return f"RMSprop with learning rate {self.lr}, beta {self.beta}, eps {self.eps}, and weight decay {self.weight_decay}."
+        return f"RMSprop with learning rate {self.lr}, beta {self.beta}, eps {self.eps}, and weight decay {self.weight_decay}."
\ No newline at end of file
