@@ -11,11 +11,14 @@ else:
     conditional_fuse = fuse
 
 @conditional_fuse
-def fused_sgd_step(param, grad, momentum, lr, beta):
-    # Update momentum and parameter in one fused kernel
-    momentum[:] = beta * momentum + (1 - beta) * grad
-    param[:] -= lr * momentum
-    return param
+def fused_update_momentum(momentum, grad, beta):
+    # momentum = beta * momentum + (1 - beta) * grad
+    return beta * momentum + (1 - beta) * grad
+
+@conditional_fuse
+def fused_param_update(param, lr, momentum):
+    # param -= lr * momentum
+    return param - lr * momentum
 
 
 class SGD(Optimizer):
@@ -44,22 +47,25 @@ class SGD(Optimizer):
         """
         for i, (name, param) in enumerate(self.params):
             if param.requires_grad and param.grad is not None:
-                
+                grad = param.grad
                 momentum_value = self.momentum[i][1]
-                     
+                
+                
                 # Weight decay fused with grad (in-place)
                 if self.weight_decay > 0:
-                    xp.add(param.grad, self.weight_decay * param.data, out=param.grad)
+                    xp.add(grad, self.weight_decay * param.data, out=grad)
                 
-                # Fused momentum + parameter update
-                param.data = fused_sgd_step(param.data, param.grad, momentum_value, self.lr, self.beta)
+                # Fused momentum update
+                momentum_value[:] = fused_update_momentum(momentum_value, grad, self.beta)
+                
+                # Fused parameter update
+                param.data[:] = fused_param_update(param.data, self.lr, momentum_value)
 
 
     def state_dict(self) -> dict:
         return {
             "lr": self.lr,
             "beta": self.beta,
-            "weight_decay": self.weight_decay,
             "params": self.params,
             "momentum": self.momentum,
         }
@@ -67,7 +73,6 @@ class SGD(Optimizer):
     def load_state_dict(self, state_dict: dict) -> None:
         self.lr = state_dict["lr"]
         self.beta = state_dict["beta"]
-        self.weight_decay = state_dict["weight_decay"]
         self.params = state_dict["params"]
         self.momentum = state_dict["momentum"]
     
